@@ -2,7 +2,7 @@
 
 from BDMLtools.clearner import outliersTransformer,dtypeAllocator,nanTransformer,dtStandardization
 from BDMLtools.report import businessReport,EDAReport,varReport,varGroupsReport,varReportSinge,varGroupsPlot,GainsTable
-from BDMLtools.selector import binSelector,binAdjuster,binFreq,binPretty,binTree,binChi2,binKmeans
+from BDMLtools.selector import binSelector,binAdjuster
 from BDMLtools.selector import faSelector
 from BDMLtools.selector import stepLogit,cardScorer
 from BDMLtools.selector import preSelector,prefitModel
@@ -11,12 +11,12 @@ from BDMLtools.selector import LgbmSeqSelector,LgbmPISelector,LgbmShapRFECVSelec
 from BDMLtools.selector import lassoSelector
 from BDMLtools.plotter import BaseWoePlotter
 from BDMLtools.plotter import perfEval,perfEval2
-from BDMLtools.encoder import woeTransformer
+from BDMLtools.encoder import woeTransformer,binTransformer
 from BDMLtools.tuner import gridTuner,hgridTuner
 from BDMLtools.tuner import BayesianCVTuner,shapCheck
 
 
-__version__ = '0.2.7'
+__version__ = '0.2.9'
 
 __all__ = (    
     dtStandardization,
@@ -33,11 +33,6 @@ __all__ = (
     GainsTable,
     binSelector,
     binAdjuster,
-    binFreq,
-    binPretty,
-    binTree,
-    binChi2,
-    binKmeans,
     BaseWoePlotter,
     faSelector,
     stepLogit,
@@ -48,6 +43,7 @@ __all__ = (
     LgbmPISelector,
     LgbmShapRFECVSelector,
     woeTransformer,
+    binTransformer,
     gridTuner,
     hgridTuner,
     BayesianCVTuner,
